#!/usr/bin/env python3
# Copyright (C) 2025 Spearhead Systems SRL

import json
from cmk.agent_based.v2 import Result, Service, State, CheckPlugin, AgentSection


# Incoming agent output is JSON of the form:
#
# {"domain": "yahoo.com", "state": "WARN", "unexpected": ["ns5.yahoo.com"], "missing": ["ns6.yahoo.com"]}
#
# Each JSON row is for a single domain. Each row always has "domain" and "state"
# fields, with optional "unexpected" and "missing" fields. The "state" field is
# one of "OK", "WARN", "CRIT", or "UNKNOWN". The "unexpected" field is a list of
# unwanted nameservers that are present in the dig query result, whereas the
# "missing" field are wanted nameservers that are not present.
#
# Return a dictionary which uses domain as key, and a dictionary of
# state/unexpected/missing.
def parse_domains_nameservers(string_table):
    results = {}

    for [line] in string_table:
        result = json.loads(line)
        results[result["domain"]] = result

    return results


def discover_domains_nameservers(section):
    for domain in section.keys():
        yield Service(item=domain)


def check_domains_nameservers(item, params, section):
    data = section.get(item)
    if not data:
        yield Result(state=State.WARN, summary="Nameservers missing")
        return

    state = State[data["state"]]
    unexpected = data.get("unexpected")
    missing    = data.get("missing")

    summary = ""

    if unexpected:
        summary += f"Unexpected nameserver(s) found: {''.join(unexpected)}. "

    if missing:
        summary += f"Expected nameserver(s) missing: {''.join(missing)}. "

    if not summary:
        summary = "All expected nameservers found, none unexpected found"

    yield Result(state=state, summary=summary)
         

agent_section_triton_wedge = AgentSection(
    name="domains_nameservers",
    parse_function=parse_domains_nameservers
)

check_plugin_domains_nameservers = CheckPlugin(
    name="domains_nameservers",
    service_name="Nameservers for Domain '%s'",

    discovery_function=discover_domains_nameservers,

    check_function=check_domains_nameservers,
    check_default_parameters={},
)
